/**
 * Assignment #9: POSIX select() and Socket Connection (Time Server)
 * Sample Solution
 *
 * Instructor: Ron Mak
 * CMPE 142 Operating Systems
 * Computer Engineering Department
 * San Jose State University
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include "globals.h"

int main(int argc, char *argv[])
{
    struct sockaddr_in server_address;
    memset(&server_address, '0', sizeof(server_address));

    // Create a socket:
    //   AF_INET: family for IPv4 addresses
    //   SOCK_STREAM: use reliable transport protocol
    //   0: let the kernel decide the default protocol
    int socket_fd = socket(AF_INET, SOCK_STREAM, 0);
    if (socket_fd < 0)
    {
        perror("*** Error: socket()");
        return -1;
    }

    // Initialize the server address.
    // htonl() and htons() convert long and short values,
    // respectively, from host byte order to network byte order.
    // INADDR_ANY: accept incoming connections across any network
    //             interface configured on the local host.
    server_address.sin_family      = AF_INET;
    server_address.sin_addr.s_addr = htonl(INADDR_ANY);
    server_address.sin_port        = htons(SERVER_PORT);

    // Bind the socket to the server address.
    if (bind(socket_fd, (struct sockaddr*) &server_address,
                        sizeof(server_address)) < 0)
    {
        perror("*** Error: bind()");
        return -1;
    }

    // Make the socket a fully functional listening socket.
    if (listen(socket_fd, MAX_CONNECTIONS) < 0)
    {
        perror("*** Error: listen()");
        return -1;
    }

    char buffer[BUFFER_SIZE];
    memset(buffer, '0', sizeof(buffer));

    while (1)
    {
        printf("Server waiting to connect ... "); fflush(stdout);

        // Accept a connection on the socket.
        int connection_fd = accept(socket_fd, (struct sockaddr*) NULL, NULL);
        if (connection_fd < 0)
        {
            perror("*** Error: accept()");
            return -1;
        }

        // Generate and send the time message over the connection.
        time_t ticks = time(NULL);
        sprintf(buffer, "%s", ctime(&ticks));
        buffer[strlen(buffer) - 1] = '\0';  // remove ending \n
        write(connection_fd, buffer, strlen(buffer) + 1);

        printf("sent %s\n", buffer);
        close(connection_fd);

        sleep(rand()%5 + 1);
     }
}
